@@ -1,9 +1,13 @@
 package Devel::DLMProf::Apache;
 our $VERSION = '0.02';
-use vars qw(%initial_modules $map $memory_before);
+use vars qw(%initial_modules $memory_before);
 use Carp;
 use Path::Class qw(file);
-use Linux::Smaps;
+
+use constant WIN32   => $^O eq 'MSWin32';
+use constant SOLARIS => $^O eq 'solaris';
+use constant LINUX   => $^O eq 'linux';
+use constant BSD_LIKE => $^O =~ /(darwin|bsd|aix)/i;
 
 BEGIN {
 
@@ -11,23 +15,39 @@ BEGIN {
     # in order that $^P settings apply to the compilation
     # of those modules.
 
-    if (!$ENV{DLMPROF}) {
+    if ( !$ENV{DLMPROF} ) {
         $ENV{DLMPROF} = "/tmp/dlmprof.$$.out";
         warn "Defaulting DLMPROF env var to '$ENV{DLMPROF}'";
     }
 
-    $map=Linux::Smaps->new($$);
+    if (LINUX) {
+
+        # need to be fixed to support older kernel
+        #unless ( eval { require Linux::Smaps } ) {
+        #    croak 'you must install Linux::Smaps';
+        #}
+    }
+    elsif (BSD_LIKE) {
+        unless ( eval { require BSD::Resource } ) {
+            croak 'you must install BSD::Resource';
+        }
+    }
+    else {
+        croak 'Not supported OS';
+    }
+
     require Devel::DLMProf;
 }
 
 use strict;
 
-use constant MP2 => (exists $ENV{MOD_PERL_API_VERSION} && $ENV{MOD_PERL_API_VERSION} == 2)
+use constant MP2 =>
+    ( exists $ENV{MOD_PERL_API_VERSION} && $ENV{MOD_PERL_API_VERSION} == 2 )
     ? 1
     : 0;
 
 sub child_init {
-    $memory_before = $map->all->size;
+    $memory_before = get_current_process_memory_size();
     $initial_modules{$_}++ for keys %INC;
 }
 
@@ -37,26 +57,76 @@ sub child_exit {
     return MP2 ? Apache2::Const::OK() : Apache::Constants::OK();
 }
 
+sub get_current_process_memory_size {
+    my $memory_size = 0;
+    if (LINUX) {
+        $memory_size = _get_linux_process_memory_size();
+    }
+    elsif (BSD_LIKE) {
+        $memory_size = _get_bsd_process_memory_size();
+    }
+    $memory_size;
+}
+
+# return process size (in KB)
+sub _get_linux_process_memory_size {
+    if ( eval { require Linux::Smaps } and Linux::Smaps->new($$) ) {
+        return _get_new_kernel_linux_process_memory_size();
+    }
+    else {
+        return _get_old_kernel_linux_process_memory_size();
+    }
+}
+
+sub _get_new_kernel_linux_process_memory_size {
+    my $map         = Linux::Smaps->new($$);
+    my $memory_size = $map->all->size;
+    $memory_size;
+}
+
+sub _get_old_kernel_linux_process_memory_size {
+    my ( $size, $resident, $share ) = ( 0, 0, 0 );
+
+    my $file = "/proc/self/statm";
+    if ( open my $fh, "<$file" ) {
+        ( $size, $resident, $share ) = split /\s/, scalar <$fh>;
+        close $fh;
+    }
+    else {
+        croak "Fatal Error: couldn't access $file";
+    }
+
+    # linux on intel x86 has 4KB page size...
+    return $size * 4;
+}
+
+sub _get_bsd_process_memory_size {
+    my @results     = BSD::Resource::getrusage();
+    my $memory_size = $results[2];
+    $memory_size;
+}
+
 sub profile_dynamic_loaded_modules {
     my @dynamic_loaded_modules = grep { !$initial_modules{$_} } keys %INC;
     my $dynamic_loaded_modules = join "¥n", @dynamic_loaded_modules;
-    _write_log("### Dynamic Loaded Modules ###\n" . $dynamic_loaded_modules);
+    _write_log(
+        "### Dynamic Loaded Modules ###\n" . $dynamic_loaded_modules );
 }
 
 sub profile_memory_diff {
-    my $map_after =Linux::Smaps->new($$);
-    my $memory_after = $map_after->all->size;
-    my $memory_diff =  $memory_after - $memory_before; 
-    _write_log("### memory (after-before) ###\n" . $memory_diff);
+    my $memory_after = get_current_process_memory_size();
+    my $memory_diff  = $memory_after - $memory_before;
+    _write_log( "### memory (after-before) ###\n" . $memory_diff );
 }
 
 sub _write_log {
     my $text = shift;
     warn $text;
-    # TODO to be fixed
-    #my $output = file($ENV{DLMPROF})->openw or croak "Can't read $ENV{DLMPROF}: $!";
-    #$output->print($text);
-    #$output->close;
+
+# TODO to be fixed
+#my $output = file($ENV{DLMPROF})->openw or croak "Can't read $ENV{DLMPROF}: $!";
+#$output->print($text);
+#$output->close;
 }
 
 # arrange for the profile to be enabled in each child
@@ -66,15 +136,15 @@ if (MP2) {
     require Apache2::ServerUtil;
     require Apache2::Const;
     my $s = Apache2::ServerUtil->server;
-    $s->push_handlers(PerlChildInitHandler => \&child_init);
-    $s->push_handlers(PerlChildExitHandler => \&child_exit);
+    $s->push_handlers( PerlChildInitHandler => \&child_init );
+    $s->push_handlers( PerlChildExitHandler => \&child_exit );
 }
 else {
     require Apache;
     require Apache::Constants;
-    if (Apache->can('push_handlers')) {
-        Apache->push_handlers(PerlChildInitHandler => \&child_init);
-        Apache->push_handlers(PerlChildExitHandler => \&child_exit);
+    if ( Apache->can('push_handlers') ) {
+        Apache->push_handlers( PerlChildInitHandler => \&child_init );
+        Apache->push_handlers( PerlChildExitHandler => \&child_exit );
     }
     else {
         Carp::carp("Apache.pm was not loaded");
@@ -104,7 +174,7 @@ If the DLMPROF environment variable isn't set I<at the time
 Devel::DLMProf::Apache is loaded> then Devel::DLMProf::Apache will issue a
 warning and default it to:
 
-	file=/tmp/dlmprof.$$.out
+	/tmp/dlmprof.$$.out
 
 Try using C<PerlPassEnv> so you can set the DLMPROF environment variable externally.
 
